@@ -1,5 +1,5 @@
 /*
- *  $Id: sctpassociation.cc,v 1.7 2003/08/19 19:20:00 tuexen Exp $
+ *  $Id: sctpassociation.cc,v 1.11 2004/07/30 11:03:43 dreibh Exp $
  *
  * SocketAPI implementation for the sctplib.
  * Copyright (C) 1999-2003 by Thomas Dreibholz
@@ -40,7 +40,7 @@
 #include "sctpsocketmaster.h"
 
 
-
+// #define PRINT_PREESTABLISHMENT_SEND
 // #define PRINT_SHUTDOWN
 // #define PRINT_SOCKTYPE
 // #define PRINT_RTOMAX
@@ -70,6 +70,9 @@ SCTPAssociation::SCTPAssociation(SCTPSocket*        socket,
    WriteReady                    = false;
    HasException                  = false;
    RTOMaxIsInitTimeout           = false;
+   FirstPreEstablishmentPacket   = NULL;
+   LastPreEstablishmentPacket    = NULL;
+   PreEstablishmentAddressList   = NULL;
 
    EstablishCondition.setName("SCTPAssociation::EstablishCondition");
    ShutdownCompleteCondition.setName("SCTPAssociation::ShutdownCompleteCondition");
@@ -155,6 +158,21 @@ SCTPAssociation::~SCTPAssociation()
       StreamDefaultTimeoutArray = NULL;
       StreamDefaultTimeoutCount = 0;
    }
+
+   PreEstablishmentPacket* packet = FirstPreEstablishmentPacket;
+   while(packet != NULL) {
+      PreEstablishmentPacket* nextPacket = packet->Next;
+      delete packet->Data;
+      packet->Data = NULL;
+      delete packet;
+      packet = nextPacket;
+   }
+   FirstPreEstablishmentPacket = NULL;
+   LastPreEstablishmentPacket  = NULL;
+   if(PreEstablishmentAddressList) {
+      SocketAddress::deleteAddressList(PreEstablishmentAddressList);
+      PreEstablishmentAddressList = NULL;
+   }
 }
 
 
@@ -180,7 +198,7 @@ bool SCTPAssociation::getRemoteAddresses(SocketAddress**& addressArray)
    if(sctp_getAssocStatus(AssociationID,&status) == 0) {
 #if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19) || (SCTPLIB_VERSION == SCTPLIB_1_0_0)
       const unsigned int addresses = status.numberOfAddresses;
-#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
       const unsigned int addresses = status.numberOfDestinationPaths;
 #else
 #error Wrong sctplib version!
@@ -193,7 +211,7 @@ bool SCTPAssociation::getRemoteAddresses(SocketAddress**& addressArray)
       for(i = 0;i < addresses;i++) {
 #if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19) || (SCTPLIB_VERSION == SCTPLIB_1_0_0)
          const int index = i;
-#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
          const int index = status.destinationPathIDs[i];
 #else
 #error Wrong sctplib version!
@@ -296,36 +314,114 @@ int SCTPAssociation::sendTo(const char*          buffer,
                             const SocketAddress* pathDestinationAddress)
 {
    int result;
-   if(!useDefaults) {
-      result = Socket->internalSend(
-                          buffer, length,
-                          flags,
-                          AssociationID, streamID, protoID,
-                          timeToLive,
-                          &ReadyForTransmit, pathDestinationAddress);
-   }
-   else {
-      if((buffer != NULL) && (length > 0)) {
-         unsigned int timeout;
-         if(!getDefaultStreamTimeout(Defaults.StreamID, timeout)) {
-            timeout = Defaults.TimeToLive;
+   if(!CommunicationUpNotification) {
+      PreEstablishmentPacket* packet = new PreEstablishmentPacket;
+      if(packet) {
+         packet->Data = new char[length];
+         if(packet->Data) {
+            memcpy(packet->Data, buffer, length);
+            packet->Length     = length;
+            packet->Next       = NULL;
+            packet->Flags      = flags;
+            packet->ProtoID    = protoID;
+            packet->StreamID   = streamID;
+            packet->TimeToLive = timeToLive;
+            if(FirstPreEstablishmentPacket == NULL) {
+               FirstPreEstablishmentPacket = packet;
+               LastPreEstablishmentPacket  = packet;
+            }
+            else {
+               LastPreEstablishmentPacket->Next = packet;
+            }
+            LastPreEstablishmentPacket       = packet;
+         }
+         else {
+            delete packet;
+            result = -ENOMEM;
          }
-         result = Socket->internalSend(
-                             buffer, length,
-                             flags,
-                             AssociationID,
-                             Defaults.StreamID, Defaults.ProtoID, Defaults.TimeToLive,
-                             &ReadyForTransmit,
-                             pathDestinationAddress);
       }
       else {
-         result = 0;
+         result = -ENOMEM;
+      }
+      result = length;
+   }
+   else {
+      if(!useDefaults) {
+         result = Socket->internalSend(
+                           buffer, length,
+                           flags,
+                           AssociationID, streamID, protoID,
+                           timeToLive,
+                           &ReadyForTransmit, pathDestinationAddress);
+      }
+      else {
+         if((buffer != NULL) && (length > 0)) {
+            unsigned int timeout;
+            if(!getDefaultStreamTimeout(Defaults.StreamID, timeout)) {
+               timeout = Defaults.TimeToLive;
+            }
+            result = Socket->internalSend(
+                              buffer, length,
+                              flags,
+                              AssociationID,
+                              Defaults.StreamID, Defaults.ProtoID, Defaults.TimeToLive,
+                              &ReadyForTransmit,
+                              pathDestinationAddress);
+         }
+         else {
+            result = 0;
+         }
       }
    }
    return(result);
 }
 
 
+// ###### Send pre-establishment packets ####################################
+bool SCTPAssociation::sendPreEstablishmentPackets()
+{
+   ssize_t result;
+
+   while(FirstPreEstablishmentPacket) {
+      SCTPAssociation::PreEstablishmentPacket* packet = FirstPreEstablishmentPacket;
+#ifdef PRINT_PREESTABLISHMENT_SEND
+      char str[256];
+      snprintf((char*)&str,sizeof(str),
+                  "A%04d: sendPreEstablishmentPackets() - Sending %u bytes, PPID $%08x, stream %u",
+                  AssociationID, packet->Length, packet->ProtoID, packet->StreamID);
+      cerr << str << endl;
+#endif
+      result = sendTo(packet->Data,
+                      packet->Length,
+                      packet->Flags,
+                      packet->StreamID,
+                      packet->ProtoID,
+                      packet->TimeToLive,
+                      false,
+                      NULL);
+      if(result == (ssize_t)packet->Length) {
+#ifdef PRINT_PREESTABLISHMENT_SEND
+         cerr << "Successfully sent packet" << endl;
+#endif
+         FirstPreEstablishmentPacket = packet->Next;
+         if(LastPreEstablishmentPacket == packet) {
+            LastPreEstablishmentPacket = NULL;
+         }
+         delete [] packet->Data;
+         packet->Data = NULL;
+         delete packet;
+      }
+      else {
+#ifdef PRINT_PREESTABLISHMENT_SEND
+         cerr << "Sending failed" << endl;
+#endif
+         return(false);
+      }
+   }
+   return(true);
+}
+
+
 // ###### Shutdown ##########################################################
 void SCTPAssociation::shutdown()
 {
@@ -343,10 +439,12 @@ void SCTPAssociation::abort()
 {
    SCTPSocketMaster::MasterInstance.lock();
    IsShuttingDown = true;
-#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
    sctp_abort(AssociationID, 0, NULL);
-#else
+#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19) || (SCTPLIB_VERSION == SCTPLIB_1_0_0)
    sctp_abort(AssociationID);
+#else
+#error Wrong sctplib version!
 #endif
    SCTPSocketMaster::MasterInstance.unlock();
 }
