@@ -1,5 +1,5 @@
 /*
- *  $Id: sctpsocket.cc,v 1.21 2003/08/19 19:20:00 tuexen Exp $
+ *  $Id: sctpsocket.cc,v 1.28 2005/03/04 13:22:14 dreibh Exp $
  *
  * SocketAPI implementation for the sctplib.
  * Copyright (C) 1999-2003 by Thomas Dreibholz
@@ -48,6 +48,7 @@
 #define PRINT_ACCEPT
 #define PRINT_ASSOCIATE
 #define PRINT_NEW_ASSOCIATIONS
+#define PRINT_SEND_TO_ALL
 #define PRINT_SHUTDOWNS
 #define PRINT_PRSCTP
 #define PRINT_NOTIFICATION_SKIP
@@ -56,6 +57,7 @@
 #define PRINT_SENDSTATUS
 #define PRINT_SETPRIMARY
 */
+
 /*
 #define PRINT_AUTOCLOSE_TIMEOUT
 #define PRINT_AUTOCLOSE_CHECK
@@ -70,7 +72,7 @@
 
 
 // ###### Constructor #######################################################
-SCTPSocket::SCTPSocket(const cardinal flags)
+SCTPSocket::SCTPSocket(const int family, const cardinal flags)
 {
    CorrelationID       = 0;
    AutoCloseTimeout    = 30000000;
@@ -82,6 +84,7 @@ SCTPSocket::SCTPSocket(const cardinal flags)
    ReadReady           = false;
    WriteReady          = false;
    HasException        = false;
+   Family              = family;
 
    EstablishCondition.setName("SCTPSocket::EstablishCondition");
    ReadUpdateCondition.setName("SCTPSocket::ReadUpdateCondition");
@@ -251,16 +254,28 @@ int SCTPSocket::bind(const unsigned short    localPort,
 
 
    // ====== Initialize local addresses =====================================
+   for(unsigned int i = 0;i < min(NoOfLocalAddresses,(unsigned int)SCTP_MAX_NUM_ADDRESSES);i++) {
+      const InternetAddress* localAddress = dynamic_cast<const InternetAddress*>(localAddressList[i]);
+      const bool isIPv6 = (localAddress != NULL) ? localAddress->isIPv6() : false;
+      if(isIPv6 && (Family == AF_INET6)) {
+         snprintf((char*)&LocalAddressList[i],SCTP_MAX_IP_LEN, "%s",
+                  localAddressList[i]->getAddressString(
+                     SocketAddress::PF_HidePort|SocketAddress::PF_Address).getData());
+      }
+      else {
+         snprintf((char*)&LocalAddressList[i],SCTP_MAX_IP_LEN, "%s",
+                  localAddressList[i]->getAddressString(
+                     SocketAddress::PF_HidePort|SocketAddress::PF_Address|
+                     SocketAddress::PF_Legacy).getData());
+      }
+   }
 #ifdef PRINT_BIND
    cout << "Binding to {";
    for(unsigned int i = 0;i < NoOfLocalAddresses;i++) {
-      cout << " " << localAddressList[i]->getAddressString(SocketAddress::PF_Address|SocketAddress::PF_HidePort|SocketAddress::PF_Legacy) << ":" << localPort << " ";
+      cout << " " << LocalAddressList[i] << " ";
    }
-   cout << "}." << endl;
+   cout << "}, port " << LocalPort << "." << endl;
 #endif
-   for(unsigned int i = 0;i < min(NoOfLocalAddresses,(unsigned int)SCTP_MAX_NUM_ADDRESSES);i++) {
-      snprintf((char*)&LocalAddressList[i],SCTP_MAX_IP_LEN,"%s",localAddressList[i]->getAddressString(SocketAddress::PF_HidePort|SocketAddress::PF_Address|SocketAddress::PF_Legacy).getData());
-   }
    if(NoOfLocalAddresses < 1) {
 #ifndef DISABLE_WARNINGS
       cerr << "ERROR: SCTPSocket::bind() - No local addresses!" << endl;
@@ -494,8 +509,16 @@ SCTPAssociation* SCTPSocket::associate(const unsigned short  noOfOutStreams,
    unsigned char addressArray[destinationAddresses][SCTP_MAX_IP_LEN];
    if(destinationAddresses > 0) {
       for(unsigned int i = 0;i < destinationAddresses;i++) {
-         snprintf((char*)&addressArray[i], SCTP_MAX_IP_LEN, "%s",
-                  destinationAddressList[i]->getAddressString(SocketAddress::PF_HidePort|SocketAddress::PF_Address|SocketAddress::PF_Legacy).getData());
+         const InternetAddress* destinationAddress = dynamic_cast<const InternetAddress*>(destinationAddressList[i]);
+         const bool isIPv6 = (destinationAddress != NULL) ? destinationAddress->isIPv6() : false;
+         if(isIPv6 && (Family == AF_INET6)) {
+            snprintf((char*)&addressArray[i], SCTP_MAX_IP_LEN, "%s",
+                     destinationAddressList[i]->getAddressString(SocketAddress::PF_HidePort|SocketAddress::PF_Address).getData());
+         }
+         else {
+            snprintf((char*)&addressArray[i], SCTP_MAX_IP_LEN, "%s",
+                     destinationAddressList[i]->getAddressString(SocketAddress::PF_HidePort|SocketAddress::PF_Address|SocketAddress::PF_Legacy).getData());
+         }
       }
 #if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19)
       assocID = sctp_associate(InstanceName,
@@ -544,10 +567,12 @@ SCTPAssociation* SCTPSocket::associate(const unsigned short  noOfOutStreams,
       association = new SCTPAssociation(this, assocID, NotificationFlags,
                                         Flags & SCTPSocket::SSF_GlobalQueue);
       if(association == NULL) {
-#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
          sctp_abort(assocID, 0, NULL);
-#else
+#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19) || (SCTPLIB_VERSION == SCTPLIB_1_0_0)
          sctp_abort(assocID);
+#else
+#error Wrong sctplib version!
 #endif
          sctp_deleteAssociation(assocID);
 #ifndef DISABLE_WARNINGS
@@ -568,6 +593,15 @@ SCTPAssociation* SCTPSocket::associate(const unsigned short  noOfOutStreams,
          association->RTOMaxIsInitTimeout = true;
          association->RTOMax              = oldParameters.rtoMax;
          association->InitTimeout         = maxInitTimeout;
+
+         association->PreEstablishmentAddressList = SocketAddress::newAddressList(destinationAddresses);
+         if(association->PreEstablishmentAddressList != NULL) {
+            for(unsigned int i = 0;i < destinationAddresses;i++) {
+               association->PreEstablishmentAddressList[i] =
+                  destinationAddressList[i]->duplicate();
+            }
+         }
+
 #ifdef PRINT_RTO
          cout << "associate() - InitTimeout=" << association->InitTimeout << " SavedMaxRTO=" << association->RTOMax << endl;
 #endif
@@ -715,7 +749,7 @@ int SCTPSocket::internalReceive(SCTPNotificationQueue& queue,
             flags |= MSG_UNORDERED;
          }
          size_t receivedBytes = min((size_t) sda->sda_bytes_arrived, (size_t) bufferSize);
-#if (SCTPLIB_VERSION != SCTPLIB_1_0_0_PRE19)
+#if (SCTPLIB_VERSION == SCTPLIB_1_0_0)
          unsigned int pathIndex;
          const int ok = sctp_receivefrom(assocID, streamID,
                                          (unsigned char*)buffer,
@@ -724,7 +758,7 @@ int SCTPSocket::internalReceive(SCTPNotificationQueue& queue,
                                          &tsn,
                                          &pathIndex,
                                          (flags & MSG_PEEK) ? SCTP_MSG_PEEK : SCTP_MSG_DEFAULT);
-#else
+#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19) || (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
          const int ok = sctp_receive(assocID, streamID,
                                      (unsigned char*)buffer,
                                      (unsigned int*)&receivedBytes,
@@ -732,6 +766,8 @@ int SCTPSocket::internalReceive(SCTPNotificationQueue& queue,
                                      &tsn,
                                      (flags & MSG_PEEK) ? SCTP_MSG_PEEK : SCTP_MSG_DEFAULT);
          const int pathIndex = sctp_getPrimary(assocID);
+#else
+#error Wrong sctplib version!
 #endif
          if(ok == 0) {
             bufferSize = receivedBytes;
@@ -939,7 +975,9 @@ int SCTPSocket::internalSend(const char*          buffer,
       }
 
 #ifdef PRINT_DATA
-      cout << "Sending data to association " << assocID << ", stream " << streamID << ", path index " << pathIndex << ":" << endl;
+      cout << "Sending " << length << " bytes of data to association "
+           << assocID << ", stream " << streamID << ", PPID "
+           << protoID << ", path index " << pathIndex << ":" << endl;
       for(size_t i = 0;i < length;i++) {
          char str[32];
          snprintf((char*)&str,sizeof(str),"%02x ",((unsigned char*)buffer)[i]);
@@ -955,7 +993,7 @@ int SCTPSocket::internalSend(const char*          buffer,
 #endif
 
 #if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19) || (SCTPLIB_VERSION == SCTPLIB_1_0_0)
-      result = sctp_send(
+      result = sctp_send_private(
                   assocID, streamID,
                   (unsigned char*)buffer, length,
                   protoID,
@@ -964,8 +1002,8 @@ int SCTPSocket::internalSend(const char*          buffer,
                   timeToLive,
                   ((flags & MSG_UNORDERED) ? SCTP_UNORDERED_DELIVERY : SCTP_ORDERED_DELIVERY),
                   ((flags & MSG_UNBUNDLED) ? SCTP_BUNDLING_DISABLED : SCTP_BUNDLING_ENABLED));
-#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
-      result = sctp_send(
+#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
+      result = sctp_send_private(
                   assocID, streamID,
                   (unsigned char*)buffer, length,
                   protoID,
@@ -1076,232 +1114,275 @@ int SCTPSocket::receiveFrom(char*           buffer,
 }
 
 
-// ###### Send ##############################################################
-int SCTPSocket::sendTo(const char*          buffer,
-                       const size_t         length,
-                       const int            flags,
-                       const unsigned int   assocID,
-                       const unsigned short streamID,
-                       const unsigned int   protoID,
-                       const unsigned int   timeToLive,
-                       const unsigned short maxAttempts,
-                       const unsigned short maxInitTimeout,
-                       const bool           useDefaults,
-                       const SocketAddress* destinationAddress,
-                       const cardinal       noOfOutgoingStreams)
+// ###### Find association for given destination address ####################
+SCTPAssociation* SCTPSocket::findAssociationForDestinationAddress(
+                    multimap<unsigned int, SCTPAssociation*>& list,
+                    const SocketAddress** destinationAddressList)
 {
-   SCTPSocketMaster::MasterInstance.lock();
-   short pathIndex = SCTP_USE_PRIMARY;
+   SCTP_PathStatus pathStatus;
+   short           pathIndex;
 
-   // ====== Check for already created association ==========================
-   SCTPAssociation* association = NULL;
-   if(destinationAddress != NULL) {
-      SCTP_PathStatus pathStatus;
-
-      if(Flags & SSF_AutoConnect) {
-         multimap<unsigned int, SCTPAssociation*>::iterator iterator =
-            ConnectionlessAssociationList.begin();
-         while(iterator != ConnectionlessAssociationList.end()) {
-            SCTP_Association_Status assocStatus;
-            if(sctp_getAssocStatus(iterator->second->AssociationID,&assocStatus) == 0) {
+   multimap<unsigned int, SCTPAssociation*>::iterator iterator = list.begin();
+   while(iterator != list.end()) {
+      SCTP_Association_Status assocStatus;
+      if(iterator->second->PreEstablishmentAddressList == NULL) {
+         if(sctp_getAssocStatus(iterator->second->AssociationID, &assocStatus) == 0) {
+            size_t i = 0;
+            while(destinationAddressList[i] != NULL) {
 #ifdef PRINT_ASSOCSEARCH
-               cout << "CL "
-                    << destinationAddress->getAddressString(InternetAddress::PF_Address|InternetAddress::PF_Legacy)
-                    << " in AssocID=" << iterator->second->AssociationID << "?" << endl;
+               cout << "Check "
+                  << destinationAddressList[i]->getAddressString(InternetAddress::PF_Address|InternetAddress::PF_Legacy)
+                  << " in AssocID=" << iterator->second->AssociationID << "?" << endl;
 #endif
                if( (!iterator->second->IsShuttingDown) &&
-                   (destinationAddress->getPort() == assocStatus.destPort) &&
-                   ((pathIndex = getPathIndexForAddress(iterator->second->AssociationID, destinationAddress, pathStatus)) >= 0) ) {
+                  (destinationAddressList[i]->getPort() == assocStatus.destPort) &&
+                  ((pathIndex = getPathIndexForAddress(iterator->second->AssociationID, destinationAddressList[i], pathStatus)) >= 0) ) {
 #ifdef PRINT_ASSOCSEARCH
                   cout << "Found: index=" << pathIndex << endl;
 #endif
-                  association = iterator->second;
-                  break;
+                  return(iterator->second);
                }
+               i++;
             }
-            iterator++;
          }
       }
-      if(association == NULL) {
-         multimap<unsigned int, SCTPAssociation*>::iterator iterator =
-            AssociationList.begin();
-         while(iterator != AssociationList.end()) {
-            SCTP_Association_Status assocStatus;
-            if(sctp_getAssocStatus(iterator->second->AssociationID,&assocStatus) == 0) {
+      else {
+         size_t i = 0;
+         size_t j = 0;
+         while(destinationAddressList[i] != NULL) {
+            while(iterator->second->PreEstablishmentAddressList[j] != NULL) {
 #ifdef PRINT_ASSOCSEARCH
-               cout << "CO "
-                    << destinationAddress->getAddressString(InternetAddress::PF_Address|InternetAddress::PF_Legacy)
-                    << " in AssocID=" << iterator->second->AssociationID << "?" << endl;
+               cout << "PreEstablishmentAddressList Check "
+                    << destinationAddressList[i]->getAddressString(InternetAddress::PF_Address|InternetAddress::PF_Legacy)
+                    << " == "
+                    << iterator->second->PreEstablishmentAddressList[j]->getAddressString(InternetAddress::PF_Address|InternetAddress::PF_Legacy) << endl;
 #endif
-               if( (!iterator->second->IsShuttingDown) &&
-                   (destinationAddress->getPort() == assocStatus.destPort) &&
-                   ((pathIndex = getPathIndexForAddress(iterator->second->AssociationID, destinationAddress, pathStatus)) >= 0) ) {
+               if(destinationAddressList[i]->getAddressString(InternetAddress::PF_Address|InternetAddress::PF_Legacy) ==
+                  iterator->second->PreEstablishmentAddressList[j]->getAddressString(InternetAddress::PF_Address|InternetAddress::PF_Legacy)) {
 #ifdef PRINT_ASSOCSEARCH
-                  cout << "Found: index=" << pathIndex << endl;
+                  cout << "Found" << endl;
 #endif
-                  association = iterator->second;
-                  break;
+                  return(iterator->second);
                }
+               j++;
             }
-            iterator++;
+            i++;
          }
       }
+      iterator++;
    }
-   else {
+   return(NULL);
+}
+
+
+// ###### Send ##############################################################
+int SCTPSocket::sendTo(const char*           buffer,
+                       const size_t          length,
+                       const int             flags,
+                       const unsigned int    assocID,
+                       const unsigned short  streamID,
+                       const unsigned int    protoID,
+                       const unsigned int    timeToLive,
+                       const unsigned short  maxAttempts,
+                       const unsigned short  maxInitTimeout,
+                       const bool            useDefaults,
+                       const SocketAddress** destinationAddressList,
+                       const cardinal        noOfOutgoingStreams)
+{
+   int result;
+
+   SCTPSocketMaster::MasterInstance.lock();
+
+   // ====== Send to one association ========================================
+   if(!(flags & MSG_SEND_TO_ALL)) {
+      // ====== Check for already created association =======================
+      SCTPAssociation* association = NULL;
+      if(destinationAddressList != NULL) {
+         if(Flags & SSF_AutoConnect) {
 #ifdef PRINT_ASSOCSEARCH
-      cout << "AssocIDLookup " << assocID << "... ";
+            cout << "Assoc lookup in ConnectionlessAssociationList..." << endl;
 #endif
-      multimap<unsigned int, SCTPAssociation*>::iterator iterator =
-         AssociationList.find(assocID);
-      if(iterator != AssociationList.end()) {
+            association = findAssociationForDestinationAddress(ConnectionlessAssociationList,
+                                                               destinationAddressList);
+         }
+         if(association == NULL) {
 #ifdef PRINT_ASSOCSEARCH
-         cout << "ok." << endl;
+            cout << "Assoc lookup in AssociationList..." << endl;
 #endif
-         association = iterator->second;
+            association = findAssociationForDestinationAddress(AssociationList,
+                                                               destinationAddressList);
+         }
       }
       else {
 #ifdef PRINT_ASSOCSEARCH
-         cout << "failed!" << endl;
+         cout << "AssocIDLookup " << assocID << "... ";
 #endif
+         multimap<unsigned int, SCTPAssociation*>::iterator iterator =
+            AssociationList.find(assocID);
+         if(iterator != AssociationList.end()) {
+#ifdef PRINT_ASSOCSEARCH
+            cout << "ok." << endl;
+#endif
+            association = iterator->second;
+         }
+         else {
+#ifdef PRINT_ASSOCSEARCH
+            cout << "failed!" << endl;
+#endif
+         }
       }
-   }
-   if(association != NULL) {
-#ifdef PRINT_ASSOC_USECOUNT
-      cout << "Send: UseCount increment for A" << association->getID() << ": "
-           << association->UseCount << " -> ";
-#endif
-      association->UseCount++;
-#ifdef PRINT_ASSOC_USECOUNT
-      cout << association->UseCount << endl;
-#endif
-   }
-
-   SCTPSocketMaster::MasterInstance.unlock();
-
-
-   // ====== Create new association =========================================
-   if((Flags & SSF_AutoConnect) && (association == NULL) && (destinationAddress != NULL)) {
-#ifdef PRINT_NEW_ASSOCIATIONS
-      cout << "AutoConnect: New outgoing association to "
-           << destinationAddress->getAddressString(InternetAddress::PF_Address|InternetAddress::PF_Legacy)
-           << "..." << endl;
-#endif
-      const SocketAddress* destinationAddressList[2];
-      destinationAddressList[0] = destinationAddress;
-      destinationAddressList[1] = NULL;
-      association = associate(noOfOutgoingStreams,
-                              maxAttempts, maxInitTimeout,
-                              (const SocketAddress**)&destinationAddressList,
-                              (flags & MSG_DONTWAIT) ? false : true);
       if(association != NULL) {
-#ifdef PRINT_NEW_ASSOCIATIONS
-         cout << "AutoConnect: New outgoing association to "
-                 << destinationAddress->getAddressString(InternetAddress::PF_Address|InternetAddress::PF_Legacy)
-                 << ", #" << association->getID() << " established!" << endl;
-#endif
-         SCTPSocketMaster::MasterInstance.lock();
 #ifdef PRINT_ASSOC_USECOUNT
-         cout << "AutoConnect: UseCount increment for A" << association->getID() << ": "
-              << association->UseCount << " -> ";
+         cout << "Send: UseCount increment for A" << association->getID() << ": "
+            << association->UseCount << " -> ";
 #endif
          association->UseCount++;
 #ifdef PRINT_ASSOC_USECOUNT
          cout << association->UseCount << endl;
 #endif
-         ConnectionlessAssociationList.insert(pair<unsigned int, SCTPAssociation*>(association->getID(),association));
-         SCTPSocketMaster::MasterInstance.unlock();
       }
+
+      SCTPSocketMaster::MasterInstance.unlock();
+
+
+      // ====== Create new association ======================================
+      if((Flags & SSF_AutoConnect) && (association == NULL) && (destinationAddressList != NULL)) {
 #ifdef PRINT_NEW_ASSOCIATIONS
-      else {
          cout << "AutoConnect: New outgoing association to "
-              << destinationAddress->getAddressString(InternetAddress::PF_Address|InternetAddress::PF_Legacy)
-              << " failed!" << endl;
-      }
+            << destinationAddressList[0]->getAddressString(InternetAddress::PF_Address|InternetAddress::PF_Legacy)
+            << "..." << endl;
 #endif
-   }
-
-
-   // ====== Send data ======================================================
-   int result;
-   if(association != NULL) {
-      if((buffer != NULL) && (length > 0)) {
-         result = association->sendTo(buffer, length, flags,
-                                      streamID, protoID, timeToLive, useDefaults,
-                                      destinationAddress);
-      }
-      else {
-         result = 0;
-      }
-
-      // ====== Remove association, if SHUTDOWN flag is set =================
-      if((flags & MSG_EOF) || (flags & MSG_ABORT)) {
-         if(flags & MSG_ABORT) {
-            association->abort();
-         }
-         else if(flags & MSG_EOF) {
-            association->shutdown();
-         }
-         if(Flags & SSF_AutoConnect) {
-#ifdef PRINT_SHUTDOWNS
-            cout << "AutoConnect: Shutdown of outgoing association ";
-            if(destinationAddress != NULL) {
-               cout << "to " << *destinationAddress << "..." << endl;
-            }
-            else {
-               cout << "A" << assocID << "..." << endl;
-            }
+         association = associate(noOfOutgoingStreams,
+                                 maxAttempts, maxInitTimeout,
+                                 destinationAddressList,
+                                 (flags & MSG_DONTWAIT) ? false : true);
+         if(association != NULL) {
+#ifdef PRINT_NEW_ASSOCIATIONS
+            cout << "AutoConnect: New outgoing association to "
+                  << destinationAddressList[0]->getAddressString(InternetAddress::PF_Address|InternetAddress::PF_Legacy)
+                  << ", #" << association->getID() << " established!" << endl;
 #endif
             SCTPSocketMaster::MasterInstance.lock();
-            multimap<unsigned int, SCTPAssociation*>::iterator iterator =
-               ConnectionlessAssociationList.find(association->getID());
-            if(iterator != ConnectionlessAssociationList.end()) {
-               ConnectionlessAssociationList.erase(iterator);
-            }
+#ifdef PRINT_ASSOC_USECOUNT
+            cout << "AutoConnect: UseCount increment for A" << association->getID() << ": "
+               << association->UseCount << " -> ";
+#endif
+            association->UseCount++;
+#ifdef PRINT_ASSOC_USECOUNT
+            cout << association->UseCount << endl;
+#endif
+            ConnectionlessAssociationList.insert(pair<unsigned int, SCTPAssociation*>(association->getID(),association));
             SCTPSocketMaster::MasterInstance.unlock();
-            delete association;
-            association = NULL;
+         }
+#ifdef PRINT_NEW_ASSOCIATIONS
+         else {
+            cout << "AutoConnect: New outgoing association to "
+               << destinationAddressList[0]->getAddressString(InternetAddress::PF_Address|InternetAddress::PF_Legacy)
+               << " failed!" << endl;
+         }
+#endif
+      }
+
+
+      // ====== Send data ===================================================
+      if(association != NULL) {
+         if((buffer != NULL) && (length > 0)) {
+            result = association->sendTo(buffer, length, flags,
+                                         streamID, protoID, timeToLive, useDefaults,
+                                         destinationAddressList ? destinationAddressList[0] : NULL);
+         }
+         else {
+            result = 0;
+         }
+
+         // ====== Remove association, if SHUTDOWN flag is set ==============
+         if((flags & MSG_EOF) || (flags & MSG_ABORT)) {
+            if(flags & MSG_ABORT) {
+               association->abort();
+            }
+            else if(flags & MSG_EOF) {
+               association->shutdown();
+            }
+            if(Flags & SSF_AutoConnect) {
 #ifdef PRINT_SHUTDOWNS
-            cout << "AutoConnect: Shutdown of outgoing association ";
-            if(destinationAddress != NULL) {
-               cout << "to " << *destinationAddress << endl;
+               cout << "AutoConnect: Shutdown of outgoing association ";
+               if(destinationAddressList != NULL) {
+                  cout << "to " << *(destinationAddressList[0]) << "..." << endl;
+               }
+               else {
+                  cout << "A" << assocID << "..." << endl;
+               }
+#endif
+               SCTPSocketMaster::MasterInstance.lock();
+               multimap<unsigned int, SCTPAssociation*>::iterator iterator =
+                  ConnectionlessAssociationList.find(association->getID());
+               if(iterator != ConnectionlessAssociationList.end()) {
+                  ConnectionlessAssociationList.erase(iterator);
+               }
+               SCTPSocketMaster::MasterInstance.unlock();
+               delete association;
+               association = NULL;
+#ifdef PRINT_SHUTDOWNS
+               cout << "AutoConnect: Shutdown of outgoing association ";
+               if(destinationAddressList != NULL) {
+                  cout << "to " << *(destinationAddressList[0]) << endl;
+               }
+               else {
+                  cout << "A" << assocID << endl;
+               }
+               cout << " completed!" << endl;
+#endif
             }
-            else {
-               cout << "A" << assocID << endl;
-            }
-            cout << " completed!" << endl;
+            checkAutoConnect();
+         }
+      }
+      else {
+         result = -EIO;
+      }
+
+
+      SCTPSocketMaster::MasterInstance.lock();
+      if(association != NULL) {
+         association->LastUsage = getMicroTime();
+         if(association->UseCount > 0) {
+#ifdef PRINT_ASSOC_USECOUNT
+            cout << "Send: UseCount decrement for A" << association->getID() << ": "
+                 << association->UseCount << " -> ";
+#endif
+            association->UseCount--;
+#ifdef PRINT_ASSOC_USECOUNT
+            cout << association->UseCount << endl;
 #endif
          }
-         checkAutoConnect();
-      }
-   }
-   else {
-      result = -EIO;
-   }
-
-
-   SCTPSocketMaster::MasterInstance.lock();
-   if(association != NULL) {
-      association->LastUsage = getMicroTime();
-      if(association->UseCount > 0) {
-#ifdef PRINT_ASSOC_USECOUNT
-      cout << "Send: UseCount decrement for A" << association->getID() << ": "
-           << association->UseCount << " -> ";
-#endif
-         association->UseCount--;
-#ifdef PRINT_ASSOC_USECOUNT
-      cout << association->UseCount << endl;
-#endif
-      }
 #ifndef DISABLE_WARNINGS
-      else {
-         cerr << "INTERNAL ERROR: SCTPSocket::sendTo() - Too many association usecount decrements!" << endl;
-         exit(1);
-      }
+         else {
+            cerr << "INTERNAL ERROR: SCTPSocket::sendTo() - Too many association usecount decrements!" << endl;
+            exit(1);
+         }
 #endif
+      }
+      SCTPSocketMaster::MasterInstance.unlock();
+
+
+      return(result);
    }
+
+   // ====== Send to all ====================================================
+   else {
+      multimap<unsigned int, SCTPAssociation*>::iterator iterator = ConnectionlessAssociationList.begin();
+      while(iterator != ConnectionlessAssociationList.end()) {
+#ifdef PRINT_SEND_TO_ALL
+         cout << "SendToAll: AssocID=" << iterator->second->AssociationID << endl;
+#endif
+         result = iterator->second->sendTo(buffer, length, flags,
+                                           streamID, protoID, timeToLive, useDefaults,
+                                           NULL);
+         iterator++;
+      }
+      result = length;
+   }
+
    SCTPSocketMaster::MasterInstance.unlock();
-
-
    return(result);
 }
 
@@ -1415,7 +1496,7 @@ int SCTPSocket::getPathIndexForAddress(const unsigned int          assocID,
       return(sctp_getPrimary(assocID));
    }
 
-#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
    SCTP_Association_Status status;
    if(sctp_getAssocStatus(assocID,&status) != 0) {
 #ifndef DISABLE_WARNINGS
@@ -1430,7 +1511,7 @@ int SCTPSocket::getPathIndexForAddress(const unsigned int          assocID,
    for(unsigned int i = 0;;i++) {
 #if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19) || (SCTPLIB_VERSION == SCTPLIB_1_0_0)
       const int index = i;
-#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
       const int index = status.destinationPathIDs[i];
 #else
 #error Wrong sctplib version!
@@ -1681,7 +1762,7 @@ bool SCTPSocket::setPeerPrimary(const unsigned int   assocID,
    unsigned char address[SCTP_MAX_IP_LEN];
    snprintf((char*)&address,sizeof(address),"%s",
             primary.getAddressString().getData());
-#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
    const int result = sctp_setRemotePrimary(assocID,address);
 #else
    const int result = -1;
@@ -1715,7 +1796,7 @@ bool SCTPSocket::addAddress(const unsigned int   assocID,
    unsigned char address[SCTP_MAX_IP_LEN];
    snprintf((char*)&address,sizeof(address),"%s",
             addAddress.getAddressString().getData());
-#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_0_0)
+#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_0_0) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
    cerr << "NOT IMPLEMENTED: sctp_addIPAddress()" << endl;
    const int result = -1;
 #else
@@ -1754,7 +1835,7 @@ bool SCTPSocket::deleteAddress(const unsigned int   assocID,
    unsigned char address[SCTP_MAX_IP_LEN];
    snprintf((char*)&address,sizeof(address),"%s",
             delAddress.getAddressString().getData());
-#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_0_0)
+#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_0_0) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
    cerr << "NOT IMPLEMENTED: sctp_deleteIPAddress()" << endl;
    const int result = -1;
 #else
