@@ -1,5 +1,5 @@
 /*
- *  $Id: sctpsocketmaster.cc,v 1.13 2003/08/19 19:20:00 tuexen Exp $
+ *  $Id: sctpsocketmaster.cc,v 1.18 2004/11/23 10:13:45 dreibh Exp $
  *
  * SocketAPI implementation for the sctplib.
  * Copyright (C) 1999-2003 by Thomas Dreibholz
@@ -110,6 +110,14 @@ SCTPSocketMaster::SCTPSocketMaster()
          return;
       }
 
+      int sd;
+      sd = socket(AF_INET, SOCK_SEQPACKET, IPPROTO_SCTP);
+      if(sd >= 0) {
+         close(sd);
+         cerr << "ERROR: Kernel SCTP seems to be available! You cannout use sctplib and kernel SCTP simultaneously!" << endl;
+         ::exit(1);
+      }
+
       InitializationResult = sctp_initLibrary();
       if(InitializationResult == 0) {
          enableOOTBHandling(false);
@@ -206,7 +214,7 @@ SCTPSocketMaster::~SCTPSocketMaster()
       // here, since sctp_abort() directly calls communicationLostNotification(),
       // which itself calls associationGarbageCollection(iterator->first,true)
       // again. Therefore, sctp_abort() is used here.
-#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
       sctp_abort(iterator->first, 0, NULL);
 #else
       sctp_abort(iterator->first);
@@ -415,7 +423,7 @@ bool SCTPSocketMaster::associationGarbageCollection(const unsigned int assocID,
 
       // ====== Delete association ==========================================
       if(sendAbort) {
-#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
          sctp_abort(assocID, 0, NULL);
 #else
          sctp_abort(assocID);
@@ -459,7 +467,7 @@ bool SCTPSocketMaster::associationGarbageCollection(const unsigned int assocID,
 
 // ###### SCTP data arrive notification callback ############################
 void SCTPSocketMaster::dataArriveNotif(unsigned int   assocID,
-                                       unsigned int   streamID,
+                                       unsigned short streamID,
                                        unsigned int   length,
                                        unsigned short ssn,
                                        unsigned int   tsn,
@@ -470,8 +478,8 @@ void SCTPSocketMaster::dataArriveNotif(unsigned int   assocID,
 #ifdef PRINT_ARRIVENOTIFICATION
    char str[256];
    snprintf((char*)&str,sizeof(str),
-               "A%04d S%02d: Data Arrive Notification, Length = %d",
-               assocID,streamID,length);
+               "A%04d S%02d: Data Arrive Notification - length=%d, PPID=%u",
+               assocID, streamID, length, protoID);
    cerr << str << endl;
 #endif
 
@@ -539,7 +547,7 @@ void SCTPSocketMaster::networkStatusChangeNotif(unsigned int   assocID,
                                                 short          destAddrIndex,
                                                 unsigned short newState,
                                                 void*          ulpDataPtr)
-#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
 void SCTPSocketMaster::networkStatusChangeNotif(unsigned int assocID,
                                                 unsigned int affectedPathID,
                                                 int          newState,
@@ -560,8 +568,10 @@ void SCTPSocketMaster::networkStatusChangeNotif(unsigned int assocID,
    SCTP_PathStatus pathStatus;
 #if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19) || (SCTPLIB_VERSION == SCTPLIB_1_0_0)
    const int ok = sctp_getPathStatus(assocID,destAddrIndex,&pathStatus);
-#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
    const int ok = sctp_getPathStatus(assocID,affectedPathID,&pathStatus);
+#else
+#error Wrong sctplib version!
 #endif
 
 
@@ -603,7 +613,7 @@ void SCTPSocketMaster::networkStatusChangeNotif(unsigned int assocID,
          case SCTP_PATH_REMOVED:
             spc->spc_state = SCTP_ADDR_REMOVED;
           break;
-#if (SCTPLIB_VERSION != SCTPLIB_1_0_0_PRE19) && (SCTPLIB_VERSION != SCTPLIB_1_0_0)
+#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
          case SCTP_ASCONF_CONFIRMED:
             spc->spc_state = SCTP_ADDR_CONFIRMED;
           break;
@@ -611,6 +621,17 @@ void SCTPSocketMaster::networkStatusChangeNotif(unsigned int assocID,
             spc->spc_state = SCTP_ADDR_CONFIRMED;
             spc->spc_error = 1;
           break;
+#elif  (SCTPLIB_VERSION == SCTPLIB_1_3_0)
+         case SCTP_ASCONF_SUCCEEDED:
+            spc->spc_state = SCTP_ADDR_CONFIRMED;
+          break;
+         case SCTP_ASCONF_FAILED:
+            spc->spc_state = SCTP_ADDR_CONFIRMED;
+            spc->spc_error = 1;
+          break;
+#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19) || (SCTPLIB_VERSION == SCTPLIB_1_0_0)
+#else
+#error Wrong sctplib version!
 #endif
          default:
             spc->spc_state = 0;
@@ -645,7 +666,7 @@ void* SCTPSocketMaster::communicationUpNotif(unsigned int   assocID,
                                              unsigned short noOfInStreams,
                                              unsigned short noOfOutStreams,
                                              int            supportPRSCTP,
-#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
                                              int            adaptationLayerIndicationLen,
                                              void*          adaptationLayerIndication,
 #endif
@@ -686,6 +707,12 @@ void* SCTPSocketMaster::communicationUpNotif(unsigned int   assocID,
          association->EstablishCondition.broadcast();
          association->WriteReady   = true;
          association->HasException = false;
+
+         if(association->PreEstablishmentAddressList) {
+            SocketAddress::deleteAddressList(association->PreEstablishmentAddressList);
+            association->PreEstablishmentAddressList = NULL;
+         }
+         association->sendPreEstablishmentPackets();
       }
       // ====== Incoming connection =========================================
       else if(socket->Flags & SCTPSocket::SSF_Listening) {
@@ -722,10 +749,12 @@ void* SCTPSocketMaster::communicationUpNotif(unsigned int   assocID,
 #ifdef PRINT_NOTIFICATIONS
          cerr << "Rejecting unwanted incoming association" << endl;
 #endif
-#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
          sctp_abort(assocID, 0, NULL);
-#else
+#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19) || (SCTPLIB_VERSION == SCTPLIB_1_0_0)
          sctp_abort(assocID);
+#else
+#error Wrong sctplib version!
 #endif
       }
 
@@ -754,7 +783,7 @@ void* SCTPSocketMaster::communicationUpNotif(unsigned int   assocID,
 void SCTPSocketMaster::communicationLostNotif(unsigned int   assocID,
                                               unsigned short status,
                                               void*          ulpDataPtr)
-#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
 void SCTPSocketMaster::communicationLostNotif(unsigned int assocID,
                                               int          status,
                                               void*        ulpDataPtr)
@@ -834,7 +863,7 @@ void SCTPSocketMaster::communicationLostNotif(unsigned int assocID,
 void SCTPSocketMaster::communicationErrorNotif(unsigned int   assocID,
                                                unsigned short status,
                                                void*          dummy)
-#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
 void SCTPSocketMaster::communicationErrorNotif(unsigned int assocID,
                                                int          status,
                                                void*        dummy)
@@ -1001,6 +1030,7 @@ void SCTPSocketMaster::queueStatusChangeNotif(unsigned int assocID,
       if(association != NULL) {
          association->ReadyForTransmit.broadcast();
          association->WriteReady = true;
+         association->sendPreEstablishmentPackets();
       }
    }
 }
@@ -1070,7 +1100,7 @@ SCTPSocket* SCTPSocketMaster::getSocketForAssociationID(const unsigned int assoc
 {
 #if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19) || (SCTPLIB_VERSION == SCTPLIB_1_0_0)
    unsigned short instanceID = 0;
-#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
    int instanceID = 0;
 #else
 #error Wrong sctplib version!
@@ -1127,7 +1157,7 @@ bool SCTPSocketMaster::initNotification(SCTPNotification& notification,
 #if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19) || (SCTPLIB_VERSION == SCTPLIB_1_0_0)
       notification.RemoteAddresses = min((unsigned short)SCTP_MAX_NUM_ADDRESSES,
                                          status.numberOfAddresses);
-#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
       notification.RemoteAddresses = min((unsigned short)SCTP_MAX_NUM_ADDRESSES,
                                          status.numberOfDestinationPaths);
 #else
@@ -1137,7 +1167,7 @@ bool SCTPSocketMaster::initNotification(SCTPNotification& notification,
          SCTP_Path_Status pathStatus;
 #if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19) || (SCTPLIB_VERSION == SCTPLIB_1_0_0)
          if(sctp_getPathStatus(assocID,i,&pathStatus) != 0) {
-#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20)
+#elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
          if(sctp_getPathStatus(assocID,status.destinationPathIDs[i],&pathStatus) != 0) {
 #else
 #error Wrong sctplib version!
